"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  createFormBuilder: () => createFormBuilder,
  useFormBuilder: () => useFormBuilder
});
module.exports = __toCommonJS(src_exports);

// src/formbuilder.tsx
var import_react = require("react");
var import_react_hook_form = require("react-hook-form");
function createFormBuilder(methods, path, key) {
  const currentPath = path.join(".");
  const cache = {};
  const { control } = methods;
  return new Proxy(
    (options) => {
      if (currentPath.length === 0) {
        throw new Error("Cannot call register at the root.");
      }
      return methods.register(currentPath, options);
    },
    {
      get(_, prop, receiver) {
        let useCached = cache[prop];
        if (useCached !== void 0) {
          return useCached;
        }
        switch (prop) {
          case Symbol.toPrimitive:
            useCached = () => currentPath;
            break;
          case "$key":
            return key ?? currentPath;
          case "$useFieldArray":
            useCached = (props) => {
              const { fields, ...rest } = (0, import_react_hook_form.useFieldArray)({
                name: currentPath,
                keyName: "$key",
                control,
                ...props
              });
              return {
                fields: fields.map(
                  ({ $key }, i) => createFormBuilder(methods, [...path, i.toString()], $key)
                ),
                ...rest
              };
            };
            break;
          case "$useController":
            useCached = (props) => (0, import_react_hook_form.useController)({
              name: currentPath,
              control,
              ...props
            });
            break;
          case "$useWatch":
            useCached = ({ name, ...rest } = {}) => (0, import_react_hook_form.useWatch)({
              name: name != null ? prependCurrentPath(currentPath, name) : currentPath === "" ? void 0 : currentPath,
              control,
              ...rest
            });
            break;
          case "$useState":
            useCached = (props) => {
              const { errors: rootErrors, dirtyFields: rootDirtyFields } = (0, import_react_hook_form.useFormState)({ ...props, control, name: currentPath });
              const errors = (0, import_react_hook_form.get)(rootErrors, currentPath);
              const dirty = (0, import_react_hook_form.get)(rootDirtyFields, currentPath);
              return { errors, dirty };
            };
            break;
          case "$setValue":
            useCached = (value) => {
              methods.setValue(currentPath, value);
            };
            break;
          case "$setError":
            useCached = (value, options) => {
              methods.setError(currentPath, value, options);
            };
            break;
          case "$discriminate":
            useCached = (fieldName) => [
              methods.watch(
                prependCurrentPath(
                  currentPath,
                  fieldName
                )
              ),
              receiver
            ];
            break;
          default:
            useCached = createFormBuilder(methods, [
              ...path,
              prop.toString()
            ]);
        }
        return cache[prop] = useCached;
      }
    }
  );
}
var prependCurrentPath = (currentPath, name) => {
  if (currentPath.length === 0) {
    return name ?? "";
  }
  if (typeof name === "string") {
    return `${currentPath}.${name}`;
  } else if (Array.isArray(name)) {
    return name.map((s) => `${currentPath}.${s}`);
  }
  return currentPath;
};
function useFormBuilder(props) {
  const methods = (0, import_react_hook_form.useForm)(props);
  const fields = (0, import_react.useMemo)(
    () => createFormBuilder(methods, []),
    [methods]
  );
  return { fields, ...methods };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createFormBuilder,
  useFormBuilder
});
